#########################################################################
# R code
# File: bp2vs1.r
#
# N is the repeat step for bootstrap and p is the confidence level.
#
# Reference: 
# Chen, H., Chong, T.T.L., and Bai, J. (2012).
#    Theory and applications of TAR model with two threshold variables.
#    Econometric Reviews, 31(2), 142-170.
#    DOI: 10.1080/07474938.2011.607100.
#########################################################################
bp2vs1<-function(x,z1,z2,regime,threshold){

N    = 500;
p    = 0.95;
null = 0;

if(max(regime)!=2){
  print("regime does match the two-regime model")
  return(0)
}

### c(1,2,2,2)
if(sum(regime==c(1,2,2,2))==4){
   n2reg1     = modelest(x,z1,z2,c(1,2,2,2),threshold);   
   n1reg      = modelest(x,z1,z2,c(1,1,1,1),threshold);
   n1lr1      = length(x)*(n1reg$rss-n2reg1$rss)/(n2reg1$rss);
   n1bpvalue1 = rep(0,N);
   for(j in 1:N){
     n1sample = x;
     l        = length(x);
     cc1234   = n1reg$cc1234;
     n1sample[cc1234] = x[cc1234]-n1reg$re1234+sample(c(n2reg1$re1,n2reg1$re234),replace=TRUE)[1:length(cc1234)];            
     sn2reg1 = modelest(n1sample,z1,z2,c(1,2,2,2),threshold);
     sn1reg  = modelest(n1sample,z1,z2,c(1,1,1,1),threshold);
     n1bp1   = length(n1sample)*(sn1reg$rss-sn2reg1$rss)/(sn2reg1$rss);
     n1bpvalue1[j] = n1bp1;
   }
             
   print("the statistics value for c(1,2,2,2) to c(1,1,1,1)")
   print(n1lr1); 
   print("the 95% bootstrap critical value:");
   print(sort(n1bpvalue1)[N*p]); 
   
   if(n1lr1<sort(n1bpvalue1)[N*p]){
     print("it is c(1,1,1,1)");
     return(0);
   }
   else{
     print("it is c(1,2,2,2")
     return(0);
   }
}   # end loop if(sum(regime==c(1,2,2,2))==4)

if(sum(regime==c(1,2,1,1))==4){
   n2reg134   = modelest(x,z1,z2,c(1,2,1,1),threshold);
   n1reg      = modelest(x,z1,z2,c(1,1,1,1),threshold);
   n1lr1      = length(x)*(n1reg$rss-n2reg134$rss)/(n2reg134$rss);
   n1bpvalue1 = rep(0,N);
   for(j in 1:N){
      n1sample = x;
      l        = length(x);
      cc1234   = n1reg$cc1234;
      n1sample[cc1234] = x[cc1234]-n1reg$re1234+sample(c(n2reg134$re134,n2reg134$re2),replace=TRUE)[1:length(cc1234)];                sn2reg134=modelest(n1sample,z1,z2,c(1,2,1,1),threshold);
      sn1reg    = modelest(n1sample,z1,z2,c(1,1,1,1),threshold);
      sn2reg134 = modelest(n1sample,z1,z2,c(1,2,1,1),threshold);
      n1bp1     = length(n1sample)*(sn1reg$rss-sn2reg134$rss)/(sn2reg134$rss);
      n1bpvalue1[j] = n1bp1;
    }
    
    print("the statistics value for c(1,2,1,1) to c(1,1,1,1)")
    print(n1lr1); 
    print("the 95% bootstrap critical value:");
    print(sort(n1bpvalue1)[N*p]); 
    
    if(n1lr1<sort(n1bpvalue1)[N*p]){
      print("it is c(1,1,1,1)");
      return(0);
    }
    else{
      print("it is c(1,2,1,1")
      return(0);
    }
}  # end loop if(sum(regime==c(1,2,1,1))==4)

if(sum(regime==c(1,1,2,1))==4){
   n2reg124   = modelest(x,z1,z2,c(1,1,2,1),threshold);
   n1reg      = modelest(x,z1,z2,c(1,1,1,1),threshold);
   n1lr1      = length(x)*(n1reg$rss-n2reg124$rss)/(n2reg124$rss);
   n1bpvalue1 = rep(0,N);
   for(j in 1:N){
      n1sample = x;
      l        = length(x);
      cc1234   = n1reg$cc1234;
      n1sample[cc1234] = x[cc1234]-n1reg$re1234+sample(c(n2reg124$re124,n2reg124$re3),replace=TRUE)[1:length(cc1234)];                sn2reg124=modelest(n1sample,z1,z2,c(1,1,2,1),threshold);
      sn1reg    = modelest(n1sample,z1,z2,c(1,1,1,1),threshold);
      sn2reg124 = modelest(n1sample,z1,z2,c(1,1,2,1),threshold);
      n1bp1     = length(n1sample)*(sn1reg$rss-sn2reg124$rss)/(sn2reg124$rss);
      n1bpvalue1[j] = n1bp1;
   }
     
   print("the statistics value for c(1,1,2,1) to c(1,1,1,1)")
   print(n1lr1); 
   print("the 95% bootstrap critical value:");
   print(sort(n1bpvalue1)[N*p]); 
    
   if(n1lr1<sort(n1bpvalue1)[N*p]){
      print("it is c(1,1,1,1)");
      return(0);
   }
   else{
      print("it is c(1,1,2,1")
      return(0);
   }
}  # end loop if(sum(regime==c(1,1,2,1))==4)

if(sum(regime==c(1,1,1,2))==4){
   n2reg123 = modelest(x,z1,z2,c(1,1,1,2),threshold);
   n1reg    = modelest(x,z1,z2,c(1,1,1,1),threshold);
   n1lr1    = length(x)*(n1reg$rss-n2reg123$rss)/(n2reg123$rss);
   n1bpvalue1 = rep(0,N);
   for(j in 1:N){
       n1sample = x;
       l        = length(x);
       cc1234   = n1reg$cc1234;
       n1sample[cc1234] = x[cc1234]-n1reg$re1234+sample(c(n2reg123$re123,n2reg123$re4),replace=TRUE)[1:length(cc1234)];                sn2reg123=modelest(n1sample,z1,z2,c(1,1,1,2),threshold);
       sn1reg    = modelest(n1sample,z1,z2,c(1,1,1,1),threshold);
       sn2reg123 = modelest(n1sample,z1,z2,c(1,1,1,2),threshold);
       n1bp1     = length(n1sample)*(sn1reg$rss-sn2reg123$rss)/(sn2reg123$rss);
       n1bpvalue1[j] = n1bp1;
    }
     
    print("the statistics value for c(1,1,1,2) to c(1,1,1,1)")
    print(n1lr1); 
    print("the 95% bootstrap critical value:");
    print(sort(n1bpvalue1)[N*p]); 
    if(n1lr1<sort(n1bpvalue1)[N*p]){
       print("it is c(1,1,1,1)");
       return(0);
    }
    else{
      print("it is c(1,1,1,2")
      return(0);
    }   
}    #   end loop if(sum(regime==c(1,1,1,2))==4)

if(sum(regime==c(1,2,2,1))==4){
   n2reg14 = modelest(x,z1,z2,c(1,2,2,1),threshold);
   n1reg   = modelest(x,z1,z2,c(1,1,1,1),threshold);
   n1lr1   = length(x)*(n1reg$rss-n2reg14$rss)/(n2reg14$rss);
   n1bpvalue1 = rep(0,N);
   for(j in 1:N){
       n1sample = x;
       l        = length(x);
       cc1234   = n1reg$cc1234;
       n1sample[cc1234] = x[cc1234]-n1reg$re1234+sample(c(n2reg14$re14,n2reg14$re23),replace=TRUE)[1:length(cc1234)];                sn2reg123=modelest(n1sample,z1,z2,c(1,1,1,2),threshold);
       sn1reg   = modelest(n1sample,z1,z2,c(1,1,1,1),threshold);
       sn2reg14 = modelest(n1sample,z1,z2,c(1,2,2,1),threshold);
       n1bp1    = length(n1sample)*(sn1reg$rss-sn2reg14$rss)/(sn2reg14$rss);
       n1bpvalue1[j] = n1bp1;
   }
     
   print("the statistics value for c(1,2,2,1) to c(1,1,1,1)")
   print(n1lr1); 
   print("the 95% bootstrap critical value:");
   print(sort(n1bpvalue1)[N*p]);
     
   if(n1lr1<sort(n1bpvalue1)[N*p]){
      print("it is c(1,1,1,1)");
      return(0);
   }
   else{
    print("it is c(1,2,2,1")
    return(0);
   } 
} #  end loop if(sum(regime==c(1,2,2,1))==4)

if(sum(regime==c(1,1,2,2))==4){
   n2reg12 = modelest(x,z1,z2,c(1,1,2,2),threshold);
   n1reg   = modelest(x,z1,z2,c(1,1,1,1),threshold);
   n1lr1   = length(x)*(n1reg$rss-n2reg12$rss)/(n2reg12$rss);
   n1bpvalue1 = rep(0,N);
   for(j in 1:N){
     n1sample = x;
     l        = length(x);
     cc1234   = n1reg$cc1234;
     n1sample[cc1234] = x[cc1234]-n1reg$re1234+sample(c(n2reg12$re12,n2reg12$re34),replace=TRUE)[1:length(cc1234)];                sn2reg123=modelest(n1sample,z1,z2,c(1,1,1,2),threshold);
     sn1reg   = modelest(n1sample,z1,z2,c(1,1,1,1),threshold);
     sn2reg12 = modelest(n1sample,z1,z2,c(1,1,2,2),threshold);
     n1bp1    = length(n1sample)*(sn1reg$rss-sn2reg12$rss)/(sn2reg12$rss);
     n1bpvalue1[j] = n1bp1;
   }
   
   print("the statistics value for c(1,1,2,2) to c(1,1,1,1)")
   print(n1lr1); 
   print("the 95% bootstrap critical value:");
   print(sort(n1bpvalue1)[N*p]); 
   if(n1lr1<sort(n1bpvalue1)[N*p]){
      print("it is c(1,1,1,1)");
      return(0);}
   else{
      print("it is c(1,1,2,2")
      return(0);
   }
} #   end loop if(sum(regime==c(1,1,2,2))==4)

if(sum(regime==c(1,2,1,2))==4){
   n2reg13 = modelest(x,z1,z2,c(1,2,1,2),threshold);
   n1reg   = modelest(x,z1,z2,c(1,1,1,1),threshold);
   n1lr1   = length(x)*(n1reg$rss-n2reg13$rss)/(n2reg13$rss);
   n1bpvalue1 = rep(0,N);
   for(j in 1:N){
     n1sample = x;
     l        = length(x);
     cc1234   = n1reg$cc1234;
     n1sample[cc1234] = x[cc1234]-n1reg$re1234+sample(c(n2reg13$re13,n2reg13$re24),replace=TRUE)[1:length(cc1234)];                sn2reg123=modelest(n1sample,z1,z2,c(1,1,1,2),threshold);
     sn1reg   = modelest(n1sample,z1,z2,c(1,1,1,1),threshold);
     sn2reg13 = modelest(n1sample,z1,z2,c(1,2,1,2),threshold);
     n1bp1    = length(n1sample)*(sn1reg$rss-sn2reg13$rss)/(sn2reg13$rss);
     n1bpvalue1[j] = n1bp1;
   }
   
   print("the statistics value for c(1,2,1,2) to c(1,1,1,1)")
   print(n1lr1); 
   print("the 95% bootstrap critical value:");
   print(sort(n1bpvalue1)[N*p]); 
   if(n1lr1<sort(n1bpvalue1)[N*p]){
      print("it is c(1,1,1,1)");
      return(0);}
   else{
      print("it is c(1,2,1,2")
      return(0);}
   }
} # end loop if(sum(regime==c(1,2,1,2))==4)