#########################################################################
# R code
# File: bp3vs.r
#
# N is the repeat step for bootstrap and p is the confidence level.
#
# Reference: 
# Chen, H., Chong, T.T.L., and Bai, J. (2012).
#    Theory and applications of TAR model with two threshold variables.
#    Econometric Reviews, 31(2), 142-170.
#    DOI: 10.1080/07474938.2011.607100.
#########################################################################
bp3vs2 <- function(x,z1,z2,regime,threshold){
N    = 500;
p    = 0.95;
null = 0;
if(max(regime)!=3){
  print("regime does match the two-regime model")
  return(0)}

if(sum(regime==c(1,2,2,3))==4){
   n3reg23 = modelest(x,z1,z2,c(1,2,2,3),threshold);
   n2reg14 = modelest(x,z1,z2,c(1,2,2,1),threshold);
   n2lr14  = length(x)*(n2reg14$rss-n3reg23$rss)/(n3reg23$rss);
   n2bpvalue14 = rep(0,N);
   for(j in 1:N){
     n2sample14 = x;
     l          = length(x);
     cc14       = n2reg14$cc14;
     cc23       = n2reg14$cc23;
     n2sample14[cc14] = x[cc14]-n2reg14$re14+sample(c(n3reg23$re1,n3reg23$re4),replace=TRUE)[1:length(cc14)];
     n2sample14[cc23] = x[cc23]-n2reg14$re23+sample(n3reg23$re23,replace=TRUE)[1:length(cc23)];
     
     sn3reg23 = modelest(n2sample14,z1,z2,c(1,2,2,3),threshold);
     sn2reg14 = modelest(n2sample14,z1,z2,c(1,2,2,1),threshold);
     n2bp14   = length(n2sample14)*(sn2reg14$rss-sn3reg23$rss)/(sn3reg23$rss);
     n2bpvalue14[j] = n2bp14;
   }
   
   print("the statistics value for c(1,2,2,3) to c(1,2,2,1)")
   print(n2lr14); 
   print("the 95% bootstrap critical value:");
   print(sort(n2bpvalue14)[N*p]); 
   
   if(n2lr14<sort(n2bpvalue14)[N*p]){
      print("it is c(1,2,2,1).  Go on to test 2 vs 1");
      return(0);}
   else{
      null     = 1;
      n2reg123 = modelest(x,z1,z2,c(1,1,1,2),threshold);
      n2lr123  = length(x)*(n2reg123$rss-n3reg23$rss)/(n3reg23$rss);
      n2bpvalue123 = rep(0,N);
      for(j in 1:N){
        n2sample123 = x;
        l           = length(x);
        cc123       = n2reg123$cc123;
        cc4         = n2reg123$cc4;
        n2sample123[cc4]   = x[cc4]-n2reg123$re4+sample(n3reg23$re4,replace=TRUE);
        n2sample123[cc123] = x[cc123]-n2reg123$re123+sample(c(n3reg23$re1,n3reg23$re23),replace=TRUE)[1:length(cc123)];
      
        sn3reg23  = modelest(n2sample123,z1,z2,c(1,2,2,3),threshold);
        sn2reg123 = modelest(n2sample123,z1,z2,c(1,1,1,2),threshold);
        n2bp123   = length(n2sample123)*(sn2reg123$rss-sn3reg23$rss)/(sn3reg23$rss);
        n2bpvalue123[j] = n2bp123;
      }
      
      print("the statistics value for c(1,2,2,3) to c(1,1,1,2)")
      print(n2lr123); 
      print("the 95% bootstrap critical value:");
      print(sort(n2bpvalue123)[N*p]); 
      
      if(n2lr123<sort(n2bpvalue123)[N*p]){
        print("it is c(1,1,1,2).  Go on to test 2 vs 1");
        return(0);}
      else{
        null   = 2;
        n2reg1 = modelest(x,z1,z2,c(1,2,2,2),threshold);
        n2lr1  = length(x)*(n2reg1$rss-n3reg23$rss)/(n3reg23$rss);
        n2bpvalue1=rep(0,N);
        for(j in 1:N){
          n2sample1 = x;
          l         = length(x);
          cc234     = n2reg1$cc234;
          cc1       = n2reg1$cc1;
          n2sample1[cc1]   = x[cc1]-n2reg1$re1+sample(n3reg23$re1,replace=TRUE);
          n2sample1[cc234] = x[cc234]-n2reg1$re234+sample(c(n3reg23$re4,n3reg23$re23),replace=TRUE)[1:length(cc234)];
              
          sn3reg23 = modelest(n2sample1,z1,z2,c(1,2,2,3),threshold);
          sn2reg1  = modelest(n2sample1,z1,z2,c(1,2,2,2),threshold);
          n2bp1    = length(n2sample1)*(sn2reg1$rss-sn3reg23$rss)/(sn3reg23$rss);
          n2bpvalue1[j]=n2bp1;
        }
        
        print("the statistics value for c(1,2,2,3) to c(1,2,2,2)")
        print(n2lr1); 
        print("the 95% bootstrap critical value:");
        print(sort(n2bpvalue1)[N*p]); 
        
        if(n2lr1<sort(n2bpvalue1)[N*p]){
          print("it is c(1,2,2,2).  Go on to test 2 vs 1");
          return(0);}
        else{
          print("it is c(1,2,2,3).");
          return(0);}
        }
      }  # end loop if(n2lr14<sort(n2bpvalue14)[N*p]) 
}        # end loop if(sum(regime==c(1,2,2,3))==4)

##### for c(1,1,2,3)

if(sum(regime==c(1,1,2,3))==4){
   n3reg12=modelest(x,z1,z2,c(1,1,2,3),threshold);
   n2reg12=modelest(x,z1,z2,c(1,1,2,2),threshold);
   n2lr12=length(x)*(n2reg12$rss-n3reg12$rss)/(n3reg12$rss);
   n2bpvalue12=rep(0,N);
   for(j in 1:N){
     n2sample12  = x;
     l           = length(x);
     cc12        = n2reg12$cc12;
     cc34        = n2reg12$cc34;
   
     n2sample12[cc12] = x[cc12]-n2reg12$re12+sample(n3reg12$re12,replace=TRUE);
     n2sample12[cc34] = x[cc34]-n2reg12$re34+sample(c(n3reg12$re3,n3reg12$re4),replace=TRUE)[1:length(cc34)];
       
     sn3reg12 = modelest(n2sample12,z1,z2,c(1,1,2,3),threshold);
     sn2reg34 = modelest(n2sample12,z1,z2,c(1,1,2,2),threshold);
     n2bp12   = length(n2sample12)*(sn2reg34$rss-sn3reg12$rss)/(sn3reg12$rss);
     n2bpvalue12[j] = n2bp12;
   }
   
   print("the statistics value for c(1,1,2,3) to c(1,1,2,2)")
   print(n2lr12); 
   print("the 95% bootstrap critical value:");
   print(sort(n2bpvalue12)[N*p]); 
   
   if(n2lr12<sort(n2bpvalue12)[N*p]){
       print("it is c(1,1,2,2).  Go on to test 2 vs 1");
       return(0);}
   else{
      null     = 1;
      n2reg123 = modelest(x,z1,z2,c(1,1,1,2),threshold);
      n2lr123  = length(x)*(n2reg123$rss-n3reg12$rss)/(n3reg12$rss);
      n2bpvalue123 = rep(0,N);
      for(j in 1:N){
        n2sample123 = x;
        l           = length(x);
        cc123       = n2reg123$cc123;
        cc4         = n2reg123$cc4;
     
        n2sample123[cc4]   = x[cc4]-n2reg123$re4+sample(n3reg12$re4,replace=TRUE);
        n2sample123[cc123] = x[cc123]-n2reg123$re123+sample(c(n3reg12$re12,n3reg12$re3),replace=TRUE)[1:length(cc123)];
         
        sn3reg12  = modelest(n2sample123,z1,z2,c(1,1,2,3),threshold);
        sn2reg123 = modelest(n2sample123,z1,z2,c(1,1,1,2),threshold);
        n2bp123   = length(n2sample123)*(sn2reg123$rss-sn3reg12$rss)/(sn3reg12$rss);
        n2bpvalue123[j] = n2bp123;
             }
    print("the statistics value for c(1,1,2,3) to c(1,1,1,2)")
    print(n2lr123); 
    print("the 95% bootstrap critical value:");
    print(sort(n2bpvalue123)[N*p]); 
    
    if(n2lr123<sort(n2bpvalue123)[N*p]){
       print("it is c(1,1,1,2).  Go on to test 2 vs 1");
       return(0);}
    else{
       null     = 2;
       n2reg124 = modelest(x,z1,z2,c(1,1,2,1),threshold);
       n2lr124  = length(x)*(n2reg124$rss-n3reg12$rss)/(n3reg12$rss);
       n2bpvalue124 = rep(0,N);
       for(j in 1:N){
         n2sample124 = x;
         l           = length(x);
         cc124       = n2reg124$cc124;
         cc3         = n2reg124$cc3;

         n2sample124[cc3]   = x[cc3]-n2reg124$re3+sample(n3reg12$re3,replace=TRUE);
         n2sample124[cc124] = x[cc124]-n2reg124$re124+sample(c(n3reg12$re12,n3reg12$re4),replace=TRUE)[1:length(cc124)];

         sn3reg12  = modelest(n2sample124,z1,z2,c(1,1,2,3),threshold);
         sn2reg124 = modelest(n2sample124,z1,z2,c(1,1,2,1),threshold);
         n2bp124   = length(n2sample124)*(sn2reg124$rss-sn3reg12$rss)/(sn3reg12$rss);
         n2bpvalue124[j] = n2bp124;
       }
       
       print("the statistics value for c(1,1,2,3) to c(1,1,2,1)")
       print(n2lr124); 
       print("the 95% bootstrap critical value:");
       print(sort(n2bpvalue124)[N*p]); 
       
       if(n2lr124<sort(n2bpvalue124)[N*p]){
         print("it is c(1,1,2,1).  Go on to test 2 vs 1");
         return(0);}
       else{
         print("it is c(1,1,2,3).");
         return(0);}
       }
    }    
}  #  end loop if(sum(regime==c(1,1,2,3))==4)

##### for c(1,2,1,3)
if(sum(regime==c(1,2,1,3))==4){
   n3reg13 = modelest(x,z1,z2,c(1,2,1,3),threshold);
   n2reg13 = modelest(x,z1,z2,c(1,2,1,2),threshold);
   n2lr13  = length(x)*(n2reg13$rss-n3reg13$rss)/(n3reg13$rss);
   n2bpvalue13 = rep(0,N);
   for(j in 1:N){
     n2sample13 = x;
     l          = length(x);
     cc24       = n2reg13$cc24;
     cc13       = n2reg13$cc13;
     n2sample13[cc24] = x[cc24]-n2reg13$re24+sample(c(n3reg13$re2,n3reg13$re4),replace=TRUE)[1:length(cc24)];
     n2sample13[cc13] = x[cc13]-n2reg13$re13+sample(n3reg13$re13,replace=TRUE);
     sn3reg13 = modelest(n2sample13,z1,z2,c(1,2,1,3),threshold);
     sn2reg13 = modelest(n2sample13,z1,z2,c(1,2,1,2),threshold);
     n2bp13   = length(n2sample13)*(sn2reg13$rss-sn3reg13$rss)/(sn3reg13$rss);
     n2bpvalue13[j] = n2bp13;
   }
   
   print("the statistics value for c(1,2,1,3) to c(1,2,1,2)")
   print(n2lr13); 
   print("the 95% bootstrap critical value:");
   print(sort(n2bpvalue13)[N*p]); 
   
   if(n2lr13<sort(n2bpvalue13)[N*p]){
      print("it is c(1,2,1,2).  Go on to test 2 vs 1");
      return(0);}
   else{
      null=1;
      n2reg123 = modelest(x,z1,z2,c(1,1,1,2),threshold);
      n2lr123  = length(x)*(n2reg123$rss-n3reg13$rss)/(n3reg13$rss);
      n2bpvalue123 = rep(0,N);
      for(j in 1:N){
        n2sample123 = x;
        l           = length(x);
        cc123 = n2reg123$cc123;
        cc4   = n2reg123$cc4;
        n2sample123[cc4]   = x[cc4]-n2reg123$re4+sample(n3reg13$re4,replace=TRUE);
        n2sample123[cc123] = x[cc123]-n2reg123$re123+sample(c(n3reg13$re13,n3reg13$re2),replace=TRUE)[1:length(cc123)];
     
        sn3reg13  = modelest(n2sample123,z1,z2,c(1,2,1,3),threshold);
        sn2reg123 = modelest(n2sample123,z1,z2,c(1,1,1,2),threshold);
        n2bp123   = length(n2sample123)*(sn2reg123$rss-sn3reg13$rss)/(sn3reg13$rss);
        n2bpvalue123[j] = n2bp123;
      }
      
      print("the statistics value for c(1,2,1,3) to c(1,1,1,2)")
      print(n2lr123); 
      print("the 95% bootstrap critical value:");
      print(sort(n2bpvalue123)[N*p]); 
      
      if(n2lr123<sort(n2bpvalue123)[N*p]){
        print("it is c(1,1,1,2).  Go on to test 2 vs 1");
        return(0);}
      else{
        null     = 2;
   	n2reg134 = modelest(x,z1,z2,c(1,2,1,1),threshold);
   	n2lr134  = length(x)*(n2reg134$rss-n3reg13$rss)/(n3reg13$rss);
   	n2bpvalue134 = rep(0,N);
   	for(j in 1:N){
 	  n2sample134 = x;
          l           = length(x);
   	  cc134       = n2reg134$cc134;
   	  cc2         = n2reg134$cc2;
          n2sample134[cc2]   = x[cc2]-n2reg134$re2+sample(n3reg13$re2,replace=TRUE);
          n2sample134[cc134] = x[cc134]-n2reg134$re134+sample(c(n3reg13$re4,n3reg13$re13),replace=TRUE)[1:length(cc134)];
   	  sn3reg13  = modelest(n2sample134,z1,z2,c(1,2,1,3),threshold);
   	  sn2reg134 = modelest(n2sample134,z1,z2,c(1,2,1,1),threshold);
   	  n2bp134   = length(n2sample134)*(sn2reg134$rss-sn3reg13$rss)/(sn3reg13$rss);
   	  n2bpvalue134[j] = n2bp134;
        }
        
        print("the statistics value for c(1,2,1,3) to c(1,2,1,1)")
        print(n2lr134); 
        print("the 95% bootstrap critical value:");
        print(sort(n2bpvalue134)[N*p]); 
        if(n2lr134<sort(n2bpvalue134)[N*p]){
          print("it is c(1,2,1,1).  Go on to test 2 vs 1");
          return(0);}
        else{
          print("it is c(1,2,1,3)");
          return(0);}
       }
    }   
} #  end loop if(sum(regime==c(1,2,1,3))==4)

### for c(1,2,3,1)
if(sum(regime==c(1,2,3,1))==4){
   n3reg14 = modelest(x,z1,z2,c(1,2,3,1),threshold);
   n2reg14 = modelest(x,z1,z2,c(1,2,2,1),threshold);
   n2lr14  = length(x)*(n2reg14$rss-n3reg14$rss)/(n3reg14$rss);
   n2bpvalue14 = rep(0,N);
   for(j in 1:N){
     n2sample14 = x;
     l          = length(x);
     cc23       = n2reg14$cc23;
     cc14       = n2reg14$cc14;
     n2sample14[cc14] = x[cc14]-n2reg14$re14+sample(n3reg14$re14,replace=TRUE);
     n2sample14[cc23] = x[cc23]-n2reg14$re23+sample(c(n3reg14$re2,n3reg14$re3),replace=TRUE)[1:length(cc23)];
     sn3reg14 = modelest(n2sample14,z1,z2,c(1,2,3,1),threshold);
     sn2reg14 = modelest(n2sample14,z1,z2,c(1,2,2,1),threshold);
     n2bp14   = length(n2sample14)*(sn2reg14$rss-sn3reg14$rss)/(sn3reg14$rss);
     n2bpvalue14[j] = n2bp14;
   }
   
   print("the statistics value for c(1,2,3,1) to c(1,2,2,1)")
   print(n2lr14); 
   print("the 95% bootstrap critical value:");
   print(sort(n2bpvalue14)[N*p]); 
   
   if(n2lr14<sort(n2bpvalue14)[N*p]){
       print("it is c(1,2,2,1).  Go on to test 2 vs 1");
       return(0);}
   else{
       null=1;
       n2reg124 = modelest(x,z1,z2,c(1,1,2,1),threshold);
       n2lr124  = length(x)*(n2reg124$rss-n3reg14$rss)/(n3reg14$rss);
       n2bpvalue124 = rep(0,N);
       for(j in 1:N){
         n2sample124 = x;
         l           = length(x);
         cc124       = n2reg124$cc124;
         cc3         = n2reg124$cc3;
         n2sample124[cc3]   = x[cc3]-n2reg124$re3+sample(n3reg14$re3,replace=TRUE);
         n2sample124[cc124] = x[cc124]-n2reg124$re124+sample(c(n3reg14$re14,n3reg14$re2),replace=TRUE)[1:length(cc124)];
         sn3reg14  = modelest(n2sample124,z1,z2,c(1,2,3,1),threshold);
         sn2reg124 = modelest(n2sample124,z1,z2,c(1,1,2,1),threshold);
         n2bp124   = length(n2sample124)*(sn2reg124$rss-sn3reg14$rss)/(sn3reg14$rss);
         n2bpvalue124[j] = n2bp124;
       }
       
    print("the statistics value for c(1,2,3,1) to c(1,1,2,1)")
    print(n2lr124); 
    print("the 95% bootstrap critical value:");
    print(sort(n2bpvalue124)[N*p]); 
    
    if(n2lr124<sort(n2bpvalue124)[N*p]){
       print("it is c(1,1,2,1).  Go on to test 2 vs 1");
       return(0);}
    else{
       null = 2;
       n2reg134 = modelest(x,z1,z2,c(1,2,1,1),threshold);
       n2lr134  = length(x)*(n2reg134$rss-n3reg14$rss)/(n3reg14$rss);
       n2bpvalue134=rep(0,N);
       for(j in 1:N){
   	 n2sample134 = x;
   	 l           = length(x);
  	 cc134       = n2reg134$cc134;
  	 cc2         = n2reg134$cc2;
 
    	 n2sample134[cc2]   = x[cc2]-n2reg134$re2+sample(n3reg14$re2,replace=TRUE);
         n2sample134[cc134] = x[cc134]-n2reg134$re134+sample(c(n3reg14$re14,n3reg14$re3),replace=TRUE)[1:length(cc134)];
   
  	 sn3reg14  = modelest(n2sample134,z1,z2,c(1,2,3,1),threshold);
  	 sn2reg134 = modelest(n2sample134,z1,z2,c(1,2,1,1),threshold);
         n2bp134   = length(n2sample134)*(sn2reg134$rss-sn3reg14$rss)/(sn3reg14$rss);
         n2bpvalue134[j] = n2bp134;
       }
       
       print("the statistics value for c(1,2,3,1) to c(1,2,1,1)")
       print(n2lr134); 
       print("the 95% bootstrap critical value:");
       print(sort(n2bpvalue134)[N*p]); 
       
       if(n2lr134<sort(n2bpvalue134)[N*p]){
         print("it is c(1,2,1,1).  Go on to test 2 vs 1");
         return(0);}
       else{
         print("it is c(1,2,3,1)");
         return(0);}
       }
    }   
}  # end loop if(sum(regime==c(1,2,3,1))==4)

### for c(1,2,3,2)
if(sum(regime==c(1,2,3,2))==4){
   n3reg24 = modelest(x,z1,z2,c(1,2,3,2),threshold);
   n2reg13 = modelest(x,z1,z2,c(1,2,1,2),threshold);
   n2lr13  = length(x)*(n2reg13$rss-n3reg24$rss)/(n3reg24$rss);
   n2bpvalue13 = rep(0,N);
   for(j in 1:N){
      n2sample13 = x;
      l          = length(x);
      cc13       = n2reg13$cc13;
      cc24       = n2reg13$cc24;
      n2sample13[cc24] = x[cc24]-n2reg13$re24+sample(n3reg24$re24,replace=TRUE);
      n2sample13[cc13] = x[cc13]-n2reg13$re13+sample(c(n3reg24$re1,n3reg24$re3),replace=TRUE)[1:length(cc13)];
      sn3reg24 = modelest(n2sample13,z1,z2,c(1,2,3,2),threshold);
      sn2reg13 = modelest(n2sample13,z1,z2,c(1,2,1,2),threshold);
      n2bp13   = length(n2sample13)*(sn2reg13$rss-sn3reg24$rss)/(sn3reg24$rss);
      n2bpvalue13[j] = n2bp13;
    }
    
    print("the statistics value for c(1,2,3,2) to c(1,2,1,2)")
    print(n2lr13); 
    print("the 95% bootstrap critical value:");
    print(sort(n2bpvalue13)[N*p]); 
    if(n2lr13<sort(n2bpvalue13)[N*p]){
       print("it is c(1,2,1,2).  Go on to test 2 vs 1");
       return(0);}
    else{
       null     = 1;
       n2reg124 = modelest(x,z1,z2,c(1,1,2,1),threshold);
       n2lr124  = length(x)*(n2reg124$rss-n3reg24$rss)/(n3reg24$rss);
       n2bpvalue124 = rep(0,N);
       for(j in 1:N){
    	 n2sample124 = x;
   	 l           = length(x);
         cc124       = n2reg124$cc124;
         cc3         = n2reg124$cc3;
         n2sample124[cc3]   = x[cc3]-n2reg124$re3+sample(n3reg24$re3,replace=TRUE);
  	 n2sample124[cc124] = x[cc124]-n2reg124$re124+sample(c(n3reg24$re24,n3reg24$re1),replace=TRUE)[1:length(cc124)];
   	 sn3reg24  = modelest(n2sample124,z1,z2,c(1,2,3,2),threshold);
  	 sn2reg124 = modelest(n2sample124,z1,z2,c(1,1,2,1),threshold);
   	 n2bp124   = length(n2sample124)*(sn2reg124$rss-sn3reg24$rss)/(sn3reg24$rss);
   	 n2bpvalue124[j] = n2bp124;
       }
       
       print("the statistics value for c(1,2,3,2) to c(1,1,2,1)")
       print(n2lr124); 
       print("the 95% bootstrap critical value:");
       print(sort(n2bpvalue124)[N*p]); 
       
       if(n2lr124<sort(n2bpvalue124)[N*p]){
         print("it is c(1,1,2,1).  Go on to test 2 vs 1");
         return(0);}
       else{
         null   = 2;
         n2reg1 = modelest(x,z1,z2,c(1,2,2,2),threshold);
         n2lr1  = length(x)*(n2reg1$rss-n3reg24$rss)/(n3reg24$rss);
         n2bpvalue1 = rep(0,N);
         for(j in 1:N){
           n2sample1 = x;
           l         = length(x);
           cc234     = n2reg1$cc234;
           cc1       = n2reg1$cc1;
           n2sample1[cc1]   = x[cc1]-n2reg1$re1+sample(n3reg24$re1,replace=TRUE);
           n2sample1[cc234] = x[cc234]-n2reg1$re234+sample(c(n3reg24$re24,n3reg24$re3),replace=TRUE)[1:length(cc234)];
           sn3reg24 = modelest(n2sample1,z1,z2,c(1,2,3,2),threshold);
           sn2reg1  = modelest(n2sample1,z1,z2,c(1,2,2,2),threshold);
           n2bp1    = length(n2sample1)*(sn2reg1$rss-sn3reg24$rss)/(sn3reg24$rss);
           n2bpvalue1[j] = n2bp1;
         }
         
         print("the statistics value for c(1,2,3,2) to c(1,2,2,2)")
         print(n2lr1); 
         print("the 95% bootstrap critical value:");
         print(sort(n2bpvalue1)[N*p]); 
         
         if(n2lr1<sort(n2bpvalue1)[N*p]){
           print("it is c(1,2,2,2).  Go on to test 2 vs 1");
           return(0);}
         else{
           print("it is c(1,2,3,2)");
         return(0);}
       }
    }   
} # end loop if(sum(regime==c(1,2,3,2))==4)

#### for c(1,2,3,3)
if(sum(regime==c(1,2,3,3))==4){
   n3reg34 = modelest(x,z1,z2,c(1,2,3,3),threshold);
   n2reg12 = modelest(x,z1,z2,c(1,1,2,2),threshold);
   n2lr12  = length(x)*(n2reg12$rss-n3reg34$rss)/(n3reg34$rss);
   n2bpvalue12 = rep(0,N);
   for(j in 1:N){
     n2sample12 = x;
     l          = length(x);
     cc12       = n2reg12$cc12;
     cc34       = n2reg12$cc34;
   
     n2sample12[cc34] = x[cc34]-n2reg12$re34+sample(n3reg34$re34,replace=TRUE);
     n2sample12[cc12] = x[cc12]-n2reg12$re12+sample(c(n3reg34$re1,n3reg34$re2),replace=TRUE)[1:length(cc12)];
     sn3reg34 = modelest(n2sample12,z1,z2,c(1,2,3,3),threshold);
     sn2reg12 = modelest(n2sample12,z1,z2,c(1,1,2,2),threshold);
     n2bp12   = length(n2sample12)*(sn2reg12$rss-sn3reg34$rss)/(sn3reg34$rss);
     n2bpvalue12[j] = n2bp12;
   }
   
   print("the statistics value for c(1,2,3,3) to c(1,1,2,2)")
   print(n2lr12); 
   print("the 95% bootstrap critical value:");
   print(sort(n2bpvalue12)[N*p]); 
   
   if(n2lr12<sort(n2bpvalue12)[N*p]){
     print("it is c(1,1,2,2).  Go on to test 2 vs 1");
     return(0);}
   else{
     null     = 1;
     n2reg134 = modelest(x,z1,z2,c(1,2,1,1),threshold);
     n2lr134  = length(x)*(n2reg134$rss-n3reg34$rss)/(n3reg34$rss);
     n2bpvalue134 = rep(0,N);
     for(j in 1:N){
       n2sample134 = x;
       l           = length(x);
       cc134       = n2reg134$cc134;
       cc2         = n2reg134$cc2;
   
       n2sample134[cc2]   = x[cc2]-n2reg134$re2+sample(n3reg34$re2,replace=TRUE);
       n2sample134[cc134] = x[cc134]-n2reg134$re134+sample(c(n3reg34$re34,n3reg34$re1),replace=TRUE)[1:length(cc134)];
       sn3reg34  = modelest(n2sample134,z1,z2,c(1,2,3,3),threshold);
       sn2reg134 = modelest(n2sample134,z1,z2,c(1,2,1,1),threshold);
       n2bp134   = length(n2sample134)*(sn2reg134$rss-sn3reg34$rss)/(sn3reg34$rss);
       n2bpvalue134[j] = n2bp134;
             }
       print("the statistics value for c(1,2,3,3) to c(1,2,1,1)")
       print(n2lr134); 
       print("the 95% bootstrap critical value:");
       print(sort(n2bpvalue134)[N*p]); 
       
       if(n2lr134<sort(n2bpvalue134)[N*p]){
         print("it is c(1,2,1,1).  Go on to test 2 vs 1");
         return(0);}
       else{
         null   = 2;
         n2reg1 = modelest(x,z1,z2,c(1,2,2,2),threshold);
         n2lr1  = length(x)*(n2reg1$rss-n3reg34$rss)/(n3reg34$rss);
         n2bpvalue1 = rep(0,N);
         for(j in 1:N){
           n2sample1 = x;
           l         = length(x);
           cc234     = n2reg1$cc234;
           cc1       = n2reg1$cc1;
   
           n2sample1[cc1]   = x[cc1]-n2reg1$re1+sample(n3reg34$re1,replace=TRUE);
           n2sample1[cc234] = x[cc234]-n2reg1$re234+sample(c(n3reg34$re34,n3reg34$re2),replace=TRUE)[1:length(cc234)];
           sn3reg34 = modelest(n2sample1,z1,z2,c(1,2,3,3),threshold);
           sn2reg1  = modelest(n2sample1,z1,z2,c(1,2,2,2),threshold);
           n2bp1    = length(n2sample1)*(sn2reg1$rss-sn3reg34$rss)/(sn3reg34$rss);
           n2bpvalue1[j] = n2bp1;
         }
         
         print("the statistics value for c(1,2,3,3) to c(1,2,2,2)")
         print(n2lr1); 
         print("the 95% bootstrap critical value:");
         print(sort(n2bpvalue1)[N*p]); 
          
         if(n2lr1<sort(n2bpvalue1)[N*p]){
           print("it is c(1,2,2,2).  Go on to test 2 vs 1");
           return(0);}
         else{
           print("it is c(1,2,3,3)");
           return(0);}
       }
    }   
}
}





   

