#########################################################################################
# R code
# File: main.txt
#
# This program is the main function, which will show you how to use this program package.
# The function thresholdest(x,z1,z2,regime,order) is for estimation of threshold varialbes
# given the model structure. 
#
# The function modelest(x,z1,z2,regime,order) is to estimate the AR order and the coefficients 
# given the structure of the threshold shift.  
# the function bp4vs3(x,z1,z2,threshold) is to test the 4-regime model vs. 3-regime model. 
# the function bp3vs2(x,z1,z2,regime,threshold) is to test the 3-regime model vs. 2-regime model. 
#
# Regime indicates the threshold shift.
# The function bp2vs1(x,z1,z2,regime,threshold) is to test a 2-regime model vs. 1-regime model.  
#
# The procedure: Given a time series data x with two threshold variable vectors z1 and z2. 
# Step 1, use thresholdest (x,z1,z2,c(1,2,3,4),c(1,1,1,1) to estimate the threshold values.  
# Step 2, use the bp-test function series to get the structure of the threshold shift, given
#         the estimated threshold value from step 1. 
# Step 3, combining the results from steps 1 and 2, use 
#         modelest(x,z1,z2,regime,threshold) to get the model estimates. 
# Step 4, refine the model by returning to step 1, with the new information about the 
#         AR order in each regime from step 3, and repeat the procedure to get a 
#         stable estimation result.
#
# Reference: 
# Chen, H., Chong, T.T.L., and Bai, J. (2012).
#    Theory and applications of TAR model with two threshold variables.
#    Econometric Reviews, 31(2), 142-170.
#    DOI: 10.1080/07474938.2011.607100.
################################################################################################

e     = rnorm(400);
z1    = rnorm(400);
z2    = rnorm(400);
x     = rep(0,400);
beta1 = 0.1;
for(i in 3:400){
  x[i] = beta1*x[i-1]+(z1[i]>0&z2[i]>0)*0.4*(x[i-1]+x[i-2])+e[i];
}


reg4=thresholdest(x,z1,z2,c(1,2,3,4),c(1,1,1,1))
$a1
(Intercept)          x1 
 0.03821354  0.55162201 

$a2
(Intercept)          x2 
 0.24888707  0.04942789 

$a3
 (Intercept)           x3 
-0.088065897 -0.005888496 

$a4
 (Intercept)           x4 
-0.005274918  0.011638332 

$threshold
[1] -0.34051295 -0.01553164

$rss
[1] 618.7532

$aic
[1] 30.86843

threshold=reg4$threshold;
bp4vs3(x,z1,z2,threshold)
[1] "the statistics value for c(1,2,3,4) to c(1,2,2,3)"
[1] 49.24451
[1] "the 95% bootstrap critical value:"
[1] 16.17735
[1] "the statistics value for c(1,2,3,4) to c(1,1,2,3)"
[1] 19.26864
[1] "the 95% bootstrap critical value:"
[1] 15.56357
[1] "the statistics value for c(1,2,3,4) to c(1,2,1,3)"
[1] 96.72964
[1] "the 95% bootstrap critical value:"
[1] 13.48632
[1] "the statistics value for c(1,2,3,4) to c(1,2,3,1)"
[1] 42.53999
[1] "the 95% bootstrap critical value:"
[1] 14.91251
[1] "the statistics value for c(1,2,3,4) to c(1,2,3,2)"
[1] 3.637944
[1] "the 95% bootstrap critical value:"
[1] 6.114824
[1] "it is c(1,2,3,2).  Go on to test 3 vs. 2"
[1] 0
> bp3vs2(x,z1,z2,c(1,2,3,2),threshold)
[1] "the statistics value for c(1,2,3,2) to c(1,2,1,2)"
[1] 96.14668
[1] "the 95% bootstrap critical value:"
[1] 10.81338
[1] "the statistics value for c(1,2,3,2) to c(1,1,2,1)"
[1] 40.90165
[1] "the 95% bootstrap critical value:"
[1] 7.037695
[1] "the statistics value for c(1,2,3,2) to c(1,2,2,2)"
[1] 52.1012
[1] "the 95% bootstrap critical value:"
[1] 7.021471
[1] "it is c(1,2,3,2)"
[1] 0

empirical studies:

x=read.table(file="e:/.../turnoverandprice1.txt",header=T);
attach(x);
n      = length(volume)
ratiop = ratiov=rep(0,n-251);
pbar1  = vbar1=rep(0,n-251);

for(i in 251:(n-1)){
  vbar = vbar1[i-250]=mean(volume[(i-150):(i-1)]);
  pbar = pbar1[i-250]=mean(close[(i-250):(i-1)]);
  ratiov[i-250] = volume[i]/vbar;
  ratiop[i-250] = close[i]/pbar;
}
return    = (close[252:n]-close[251:(n-1)])/close[251:(n-1)];
threshold = thresholdest2(return,ratiop,ratiov);
